#!/usr/bin/env python
# -*- coding: utf-8 -*-

import string

from cgauser import *
from cgauserconfig import *

import pygtk

pygtk.require("2.0")

import gtk
import gtk.glade

class CgaPasswordError(Exception): pass
class CgaEmptyFields(Exception): pass
class CgaInvalidCharacter(Exception): pass

MSGERR=gtk.MESSAGE_ERROR
MSGINF=gtk.MESSAGE_INFO
VALID_CHARS=string.ascii_letters + "0123456789"
VALID_CHARS_EXTENDED=VALID_CHARS + "ñ" + "áéíóú" + "ç" + "-_." + " "

class CgaNewUserGUI():
    def __init__(self):
        self.xml=gtk.glade.XML(GLADE_FILE)
        if self.xml is None:
            raise Exception("Error al leer el XML (%s)"% GLADE_FILE)
        
        acciones={  "on_texto_usuario_changed": self.update_id,
                    "on_b_aceptar_clicked": self.create_user,
                    "on_b_cancelar_clicked": gtk.main_quit,
        }

        self.xml.signal_autoconnect(acciones)

        self.window=self.xml.get_widget("CNUWindow")
        self.t_usuario=self.xml.get_widget("texto_usuario")
        self.t_id=self.xml.get_widget("texto_id")
        self.t_pwd1=self.xml.get_widget("texto_pwd1")
        self.t_pwd2=self.xml.get_widget("texto_pwd2")
        self.b_aceptar=self.xml.get_widget("b_aceptar")

        # Se hace que el botón de aceptar acepte la acción por defecto
        self.b_aceptar.set_flags(gtk.CAN_DEFAULT)

        # Se indica que la acción por defecto es pulsar el boton aceptar
        self.window.set_default(self.b_aceptar)

        # En este caso, todos los campos de texto activan la acción por defecto.
        self.t_usuario.set_activates_default(True)
        self.t_id.set_activates_default(True)
        self.t_pwd1.set_activates_default(True)
        self.t_pwd2.set_activates_default(True)

        self.window.connect("delete-event", gtk.main_quit)

        self.window.show()

    def update_id(self, widget):
        usuario=widget.get_text()
        prop=""
        for trozo in usuario.split(" "):
            prop+=trozo[:3]

        self.t_id.set_text(prop.lower())

    def create_user(self, boton):
        usuario=self.t_usuario.get_text()
        login=self.t_id.get_text()
        pwd1=self.t_pwd1.get_text()
        pwd2=self.t_pwd2.get_text()
        mensaje=None
        try: 
            self._data_ok(usuario, login, pwd1, pwd2)
            self.is_sane(usuario)
            self.is_sane(pwd1)
            self.is_sane(pwd2)
            self.is_sane(login, restricted=True)
        except CgaInvalidCharacter, e:
            mensaje="""Uno de los campos contiene caracteres no válidos. Los caracteres válidos para cada campo son:
* Nombre y Contraseña: 
    '%s'
* Login: 
    '%s'""" % (VALID_CHARS_EXTENDED, VALID_CHARS)
        except CgaEmptyFields:
            mensaje="Todos los campos son obligatorios.\nPor favor compruebe que ha rellenado todos los campos."
        except CgaPasswordError:
            mensaje="Las contraseñas son distintas.\nPor favor asegúrese de que escribe la contraseña correctamente en los dos campos"

        if mensaje is not None:
            self.msgbox(MSGERR, mensaje)
            return

#        print """Creando el usuario:
 #           Usuario: {usuario}
  #          Login: {id}
   #         Contraseña: {pwd}""".format(usuario=usuario, id=login, pwd=pwd2)
    
        error = None
        try:
            CgaUser(login, usuario, pwd1).save()
        except CgaUserExists, msg:
            print "El usuario ya existe"
            error = msg
        except CgaUserCreationError, msg:
            print "Ha ocurrido un error al crear el usuario"
            error = msg

        if error:
            msg="No se pudo crear el usuario '%s':\n'%s'" % (login, msg)
            self.msgbox(MSGERR, msg)
        else:
            self.msgbox(MSGINF, "Usuario '%s' creado correctamente." % login)
            gtk.main_quit()

    def msgbox(self, tipo, msg):
        m=gtk.MessageDialog(parent=self.window, 
                            flags=gtk.DIALOG_MODAL, 
                            type=tipo, 
                            buttons=gtk.BUTTONS_OK,
                            message_format=msg)
        m.set_markup(msg)
        m.run()
        m.destroy()

    def _data_ok(self, usuario, login, pwd1, pwd2):
        if usuario == "" or login =="" or pwd1=="" or pwd2 =="":
            raise CgaEmptyFields
        if pwd1!=pwd2:
            raise CgaPasswordError

    def is_sane(self, cad, restricted=False):
        valid = (VALID_CHARS_EXTENDED, VALID_CHARS)[restricted]
        res = map(lambda k: k in valid, str(cad))
        if not all(res):
            raise CgaInvalidCharacter(cad)

if __name__=="__main__":
    c = CgaNewUserGUI()
    gtk.main()
