#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import pwd
import datetime
from crypt import crypt
import commands

from cgauserconfig import CUSTOMUSERSFILE
from cgauserconfig import DEFAULTGROUPS

class MissingUserData(Exception): pass
class CgaUserExists(Exception): pass
class CgaUserCreationError(Exception): pass

class CgaUser:
    def __init__(self, login, nombre, passwd):
        self.id, self.nombre, self.password = login, nombre, passwd
    
    def save(self):
        self._save()

    def _save(self):
        self._add_to_system()
        self._add_to_list()
            
    def _get_new_uid(self):
        validos = [ t.pw_uid for t in pwd.getpwall() if 1000 <= t.pw_uid < 10000 ]
        validos.sort()
        return validos[-1]+1

    def _add_to_system(self):
        try:
            user=pwd.getpwnam(self.id)
            raise CgaUserExists("El usuario %s ya existe" % self.id)
        except KeyError:
            pass
        # El usuario no existe, podemos proceder.
        
        # Primero se cifra el password para pasarlo a adduser
        passwd = crypt(self.password, "00")
        uid=self._get_new_uid()

        comando = "/usr/sbin/useradd -m -G {grupos} -p {password} -u {uid} -U -s /bin/bash {usuario}"
        comando = comando.format(   grupos=DEFAULTGROUPS,  
                                    password=passwd, 
                                    usuario=self.id,
                                    uid=uid)
        #print "Ejecutando '%s'" % comando
        (ret, salida)=commands.getstatusoutput(comando)
        comando=None
        if ret != 0:
            raise CgaUserCreationError("Error al crear el usuario: '%s'" % salida)
            
        comando="/usr/bin/chfn -f '{nombre}' {login}".format(nombre=self.nombre, login=self.id)
        (ret, salida)=commands.getstatusoutput(comando)
        comando=None
        if ret!=0:
            # Si el comando anterior falla tampoco pasa nada, es información optativa
            print "No se pudo establecer el GECOS: %s" % salida

    def _add_to_list(self):
        usuario = os.environ["LOGNAME"]
        fecha=str(datetime.datetime.now())
        with open(CUSTOMUSERSFILE, "a") as f:
            f.write("%s:%s:%s\n" % (usuario, fecha, self.id))

if __name__=="__main__":
    c = CgaUser()
    c.save()


