#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

from cgauser import *

import unittest

class TestCgaUser(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        os.seteuid(0)

    def testDefaultArgs(self):
        """ Prueba los argumentos por defecto de la clase """
        c = CgaUser()
        self.assertRaises(MissingUserData, c.save)

    def testUserExists(self):
        """ Comprueba que se produce un error si se intenta crear un usuario ya existente """
        c = CgaUser(id="derosa", 
                    nombre="David Erosa García", 
                    passwd="Petete")
        self.assertRaises(CgaUserExists, c.save)

    def testCreationError(self):
        """ Comprueba que se produce un error si el usuario no tiene permisos para crear usuarios"""
        # Se establece el euid 1000 para que no se pueda crear el usuario por
        # permisos.
        os.seteuid(1000)
        c = CgaUser(id="testuser1", 
                    nombre="Usuario de prueba 1", 
                    passwd="testuser1")
        self.assertRaises(CgaUserCreationError, c.save)

    def testUserCreated(self):
        """ Comprueba que se crea el nuevo usuario correctamente"""
        username="testuser2"
        os.system("userdel %s" % username)
        c = CgaUser(id=username,
                    nombre="Usuario de prueba 2",
                    passwd="testuser2")
        c.save()

        from pwd import getpwnam
        try:
            p = getpwnam(username)
        except KeyError, msg:
            self.fail(msg)

if __name__=="__main__":
    unittest.main()
